## Copyright (C) 2016-2019 Markus Muetzel
##
## This file is part of Octave.
##
## Octave is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <https://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {} {[@var{faces}, @var{vertices}, @var{J}] =} __unite_shared_vertices__ (@var{faces}, @var{vertices})
##
## Detect and unite shared vertices in patches.
##
## Vertices of neighboring faces are detected and united to shared vertices.
## For this, the mutual squared distances between all vertices are
## calculated.  If all coordinates are closer than
## @code{2 * eps (max (abs (vertices(:))))}, the vertices are united to one.
##
## @var{J} holds the indices of the deleted vertices.
##
## @seealso{isosurface, reducepatch}
## @end deftypefn

## Author: mmuetzel

function [faces, vertices, J] = __unite_shared_vertices__ (faces, vertices)

  lut = (1:size (vertices,1))';
  J = lut;
  for di=1:3
    [v,i] = sortrows (vertices,1+mod ((0:2)+di,3));
    Js = [true; any(abs (diff (v,1,1)) > eps (1.0) * abs (v(2:end,:)+v(1:end-1,:)),2)];
    vertices = v(Js,:);
    J = J(i(Js));
    l(i) = cumsum (Js);
    lut = l(lut);
  endfor
  [J,i] = sort (J);
  j(i) = 1:length (i);
  vertices = vertices(i,:);
  faces = j(lut(faces));
  faces = faces(all (faces-faces(:,[2 3 1])!=0,2),:);
  
endfunction
